<?php

/*
Plugin Name: Knspr-ImgNote
Plugin URI: http://knuspermagier.de
Description:  Put flickr-like notes on images.
Author: knuspermagier
Version: 1.0-final
Author URI: http://knuspermagier.de
*/

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

$noteFeedMessage = get_option('kim_feed_message', -1) ? '<p>This image contains notes, please open the website directly</p>' : get_option('kim_feed_message');
$noteShowNoteCount = get_option('kim_show_note_count', -1) == -1 ? true : get_option('kim_show_note_count');
$noteTheme = get_option('kim_note_theme', -1) == -1 ? 'default' : get_option('kim_note_theme');

function vdbg($v) {
    echo '<pre>';
    var_dump($v);
    echo '</pre>';
}

require_once('knspr-findpaths.php');
require_once('lib/knspr-imgnote.php');
require_once('KnsprNoteWordpressSaveStrategy.php');

$ajaxUrl = WP_PLUGIN_URL . '/knspr-imgnote/knspr-imgnote-ajax.php';

function knspr_add_header_script() {
    global $noteTheme;

    wp_enqueue_script('jquery');
    wp_enqueue_script('imgnotes', WP_PLUGIN_URL .'/knspr-imgnote/lib/scripts/jquery.imgnotes-0.2.js', array('jquery'));
    wp_enqueue_script('knsprimgnotes',WP_PLUGIN_URL .'/knspr-imgnote/lib/scripts/knspr-imageNote.js', array('jquery', 'imgnotes'));

    if(is_admin()) {
        wp_enqueue_script('imgareaselect', WP_PLUGIN_URL . '/knspr-imgnote/lib/scripts/jquery.imgareaselect.js', array('jquery'));
    }
}

function knspr_add_header_styles() {
    global $noteTheme;

    if(is_admin()) {
        echo '<link rel="stylesheet" type="text/css" href="'. WP_PLUGIN_URL .'/knspr-imgnote/lib/css/knspr-imagenote.css" />';
        echo '<link rel="stylesheet" type="text/css" href="'. WP_PLUGIN_URL .'/knspr-imgnote/lib/css/imgareaselect-animated.css" />';
    }

    echo '<link rel="stylesheet" type="text/css" href="'. WP_PLUGIN_URL .'/knspr-imgnote/themes/' . $noteTheme .'/imgnotes.css" />';
}

function knspr_add_image_choose_thingy() {
    global $post, $ajaxUrl;

    if(function_exists('add_meta_box')) {
        add_meta_box('knsprImgNote', 'Edit image notes', 'knspr_create_image_chooser', 'post');
        add_meta_box('knsprImgNote', 'Edit image notes', 'knspr_create_image_chooser', 'page');
    } else {
        add_action('dbx_post_advanced', 'knspr_create_image_chooser_with_box');
        add_action('dbx_page_advanced', 'knspr_create_image_chooser_with_box');
    }

}

function knspr_create_image_chooser() {
    global $post, $ajaxUrl;

    $blogUrl = WP_PLUGIN_URL;

    echo <<<EOF
			<script type="text/javascript">
            knsprImageNote.ajaxPostUrl = '$ajaxUrl';
            knsprImageNote.loadThumbs({$post->ID});
	</script>
	
		
		<p class="reload-thumbs-link"><a href="javascript:void(0);"  onclick="knsprImageNote.loadThumbs({$post->ID});"><img src="$blogUrl/knspr-imgnote/lib/css/reload.png" alt="reload" title="Refresh thumbnails"/> Reload thumbnails</a></p><div id="imageThumbs"></div>
			
		<div id="imgNoteAjaxResponse"></div>
		<img src="$blogUrl/knspr-imgnote/lib/css/spinner.gif" alt="loading" id="ajaxSpinner" style="display: none;"/>

EOF;

}

function knspr_create_image_chooser_with_box() {
    global $post, $ajaxUrl;

    $blogUrl = get_bloginfo('siteurl');

    echo <<<EOF

	<div id="imageNoteManager" class="postbox">
		<h3 class="hndle"><span>Edit image notes</span></h3>
		<div class="inside">
EOF;

    knspr_create_image_chooser();

    echo <<<EOF
		
		</div>
	</div>
EOF;
}

/**
 * Use this if you have a title picture stored as an URL in a custom field
 * <?php echo knspr_get_frontend_custom_field($post->ID, 'myCustomField', '#postTitlePicture-' . $post->ID); ?>
 * @param $postId Id of the post
 * @param $name Name of the custom field
 * @param $imageSelector Name of a unique selector to select the image via javascript
 */
function knspr_get_frontend_custom_field($postId, $name, $imageSelector) {
    $url = get_post_meta($postId, $name, true);

    if($url == '') {
        return;
    }

    $images = get_posts(array('post_type' => 'attachment',
        'numberposts' => -1, 'post_status' => null, 'post_parent' => $postid));

    if($images == false || !count($images)) {
        return;
    }

    foreach($images as $image) {
        $iurl = wp_get_attachment_image_src($image->ID, 'full');

        if($iurl[0] == $url) {
            $mgr = new KnsprNoteManager();
            $mgr->setStrategy(new KnsprNoteWordpressSaveStrategy($image->ID));
            return knspr_get_frontend_code_custom_class($mgr, $image->ID, $imageSelector);
        }
    }

    return;
}

function knspr_get_frontend_code(KnsprNoteManager $manager, $imageId) {
    return knspr_get_frontend_code_custom_class($manager, $imageId, '.wp-image-' . $imageId);
}

function knspr_get_frontend_code_custom_class(KnsprNoteManager $manager, $imageId, $cssClass) {
    global $noteShowNoteCount;

    $json =  $manager->printNoteJson(true);
    $notes = $manager->getCount();

    if($notes == 0) {
        return;
    }

    $content = '';


    if(preg_match('/iphone/i', $_SERVER['HTTP_USER_AGENT'])) {
        $content .= <<<EOF
<p class="note-iphonebutton"><a href="javascript:void(0)" onclick="knsprImageNote.showAllNotes($imageId);">Show all notes</a></p>
EOF;
    }

    if($noteShowNoteCount == true) {
        $content .= <<<EOF
	<p class="numberOfNotes">Notes: $notes</p>
EOF;
    }


    $imgData = wp_get_attachment_image_src($imageId, 'full');

    $content .= <<<EOF
        <script type="text/javascript">
jQuery(function() {
    knsprImageNote.initializeNoteDisplay('$cssClass', $json, $imageId, false, false, $imgData[1], $imgData[2]);
});
</script>
EOF;

    return $content;
}

function knspr_user_view_thingy($content) {
    global $noteFeedMessage;

    preg_match_all('/wp-image-([0-9]+)/i', $content, $matches);
    $count = count($matches[0]);

    $manager = new KnsprNoteManager();

    for($i = 0; $i < $count; $i++) {
        $manager->setStrategy(new KnsprNoteWordpressSaveStrategy($matches[1][$i]));

        if(is_feed()) {
            if($manager->getCount() != 0) {
                $content = knspr_find_image_and_append_text('wp-image-' . $matches[1][$i], $content, $noteFeedMessage);
            }
        }

        $content = knspr_find_image_and_append_text('wp-image-' . $matches[1][$i], $content, knspr_get_frontend_code($manager, $matches[1][$i]));
    }

    return $content;

}

function knspr_find_image_and_append_text($class, $content, $text) {
    $pos = strpos($content, $class);

    if($pos === false) {
        return $content;
    }

    for($i = $pos; $i < strlen($content); $i++) {
        if($content[$i] == '>') {
            return substr($content, 0, $i+1) . $text . substr($content, $i+1);
        }
    }
}

function knspr_imgnote_menu() {
    add_submenu_page('plugins.php', 'knspr-ImgNote', 'knspr-ImgNote', 'manage_options', 'knspr-imgnote-options', 'knspr_imgnote_options');
    add_action('admin_init', 'knspr_imgnote_register_options');
}

function knspr_imgnote_register_options() {
    register_setting('kim-settings-group', 'kim_feed_message');
    register_setting('kim-settings-group', 'kim_show_note_count');
    register_setting('kim-settings-group', 'kim_note_theme');
}

function knspr_imgnote_options() {
    global $noteFeedMessage, $noteShowNoteCount, $noteTheme;

    $option_feed_message = get_option('kim_feed_message', -1) == -1 ? $noteFeedMessage : get_option('kim_feed_message');
    $option_show_note_count = get_option('kim_show_note_count', -1) == -1 ? $noteShowNoteCount : get_option('kim_show_note_count');
    $option_note_theme = get_option('kim_note_theme', -1) == -1 ?  $noteTheme : get_option('kim_note_theme');

    $ncchecked = $option_show_note_count == true ? 'checked="checked"' : '';

    echo <<<EOF
    <div class="wrap">
<h2>knspr-ImgNote Options</h2>
<form method="post" action="options.php">
EOF;

    settings_fields('kim-settings-group');

    echo <<<EOF
<table class="form-table">

<tr valign="top">
<th scope="row">Message shown in the RSS feed.</th>
<td><input type="text" name="kim_feed_message" value="$option_feed_message" /></td>
</tr>

<tr valign="top">
<th scope="row">Show note count?</th>
<td><input type="checkbox" name="kim_show_note_count" value="1" $ncchecked/></td>
</tr>

<tr valign="top">
<th scope="row">Theme to use</th>
<td><input type="text" name="kim_note_theme" value="$option_note_theme" /></td>
</tr>

</table>
<input type="hidden" name="action" value="update" />
<input type="hidden" name="page_options" value="kim_feed_message,kim_show_note_count,kim_note_theme" /
<p class="submit">
<input type="submit" class="button-primary" value="Save Changes" />
</p>
</form>
</div>

EOF;
}

add_action('admin_menu', 'knspr_imgnote_menu');

add_filter('the_content', 'knspr_user_view_thingy');

add_action('init', 'knspr_add_header_script');
add_action('wp_print_styles', 'knspr_add_header_styles');
add_action('admin_print_styles', 'knspr_add_header_styles');

add_action('admin_menu', 'knspr_add_image_choose_thingy');
